local M = {}

local scenario = require("scenario/scenarios")
local helper = require("scenario/scenariohelper")

local player = {}
local suspects = {}
local text = {
	win = "Suspects under custody. Good job, Officer!",
	damage = "You got wrecked!"
}

local mainTimer = 0
local delay = 0
local score = 0
local tick = 0.25
local racerAggro = 1.25
local stuckSpeed = 3
local arrestTime = 5
local arrestRadius = 30
--local damageLimit = 100000000000000000000000000000000000

local function arrestSuspect(v)
	if v.state then
		if v.arrestTimer % 1 == 0 then
			local countdown = arrestTime - math.modf(v.arrestTimer)
			if countdown > 0 then
				helper.flashUiMessage((tostring(countdown).."..."), 0.5, true)
			end
		end
		v.arrestTimer = v.arrestTimer + tick
	else 
		v.arrestTimer = 0
	end 	
	if v.arrestTimer > arrestTime then
		helper.flashUiMessage("Busted!", 2, true)
		v.obj:queueLuaCommand("ai.setMode('stop')")
		v.arrestTimer = 0
		score = score + 1
		v.state = false
	end
end

local function onScenarioUIReady()
	player = {}
	suspects = {}
	mainTimer = 0
	delay = 0
	score = 0
	player.obj = be:getPlayerVehicle(0)
	player.id = player.obj:getID()
	local others = core_multiSpawn.getOtherVehicles()
	for i, v in ipairs(others) do
		local a = {}
		a.obj = scenetree.findObjectById(v)
		a.id = v
		a.arrestRadius = arrestRadius
		a.state = true
		table.insert(suspects, a)
	end
end

local function onRaceStart()
	for i, v in ipairs (suspects) do
		v.obj:queueLuaCommand("ai.setAggression("..tostring(racerAggro)..")")
		v.obj:queueLuaCommand("ai.setMode('flee')")
	end
end

local function onRaceTick(raceTickTime)
	player.pos = vec3(player.obj:getPosition())
	player.vel = map.objects[player.id].vel:length()
	player.damage = map.objects[player.id].damage
	for i, v in ipairs(suspects) do
		v.pos = vec3(v.obj:getPosition())
		v.vel = map.objects[v.id].vel:length()
		v.dist = v.pos:distance(player.pos)
		if v.state then
			if v.vel <= 3 then 
				v.arrestRadius = math.min(100, v.arrestRadius + 1)
			else
				v.arrestRadius = arrestRadius
			end
			if mainTimer >= 3 and v.dist <= v.arrestRadius and v.vel <= stuckSpeed then
				arrestSuspect(v)
			else
				v.arrestTimer = 0
			end
		end
	end
	--if player.damage >= damageLimit then
		--scenario.finish({failed = text.damage})
	--end
	if score >= #suspects then
		if delay == 0 then
			delay = mainTimer + 2
		end
		if mainTimer >= delay then
			scenario.finish({msg = text.win})
		end
	end
	mainTimer = mainTimer + tick
end

M.onScenarioUIReady = onScenarioUIReady
M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick

return M